# Copyright (c) 2022, Bamboooz
# All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import ctypes


class SYSTEM_POWER_STATUS(ctypes.Structure):
    _fields_ = [
        ('ACLineStatus', ctypes.c_byte),
        ('BatteryFlag', ctypes.c_byte),
        ('BatteryLifePercent', ctypes.c_byte),
        ('Reserved1', ctypes.c_byte),
        ('BatteryLifeTime', ctypes.c_ulong),
        ('BatteryFullLifeTime', ctypes.c_ulong),
    ]

def get_battery_info():
    # Load the DLL
    try:
        battery_dll = ctypes.windll.LoadLibrary('kernel32.dll')
        power_dll = ctypes.windll.LoadLibrary('PowrProf.dll')
    except OSError:
        return None

    # Call the GetSystemPowerStatus function
    system_power_status = SYSTEM_POWER_STATUS()

    if not battery_dll.GetSystemPowerStatus(ctypes.pointer(system_power_status)):
        return None

    # Call the CallNtPowerInformation function
    class SYSTEM_BATTERY_STATE(ctypes.Structure):
        _fields_ = [
            ('AcOnLine', ctypes.c_ulong),
            ('BatteryPresent', ctypes.c_ulong),
            ('Charging', ctypes.c_ulong),
            ('Discharging', ctypes.c_ulong),
            ('Spare1', ctypes.c_ulong),
            ('Spare2', ctypes.c_ulong),
            ('Spare3', ctypes.c_ulong),
            ('Spare4', ctypes.c_ulong),
            ('MaxCapacity', ctypes.c_ulong),
            ('RemainingCapacity', ctypes.c_ulong),
            ('Rate', ctypes.c_long),
            ('EstimatedTime', ctypes.c_long),
            ('DefaultAlert1', ctypes.c_ulong),
            ('DefaultAlert2', ctypes.c_ulong),
        ]

    class SYSTEM_POWER_INFORMATION(ctypes.Structure):
        _fields_ = [
            ('MaxIdlenessAllowed', ctypes.c_ulong),
            ('Idleness', ctypes.c_ulong),
            ('TimeRemaining', ctypes.c_ulong),
            ('CoolingMode', ctypes.c_ulong),
        ]

    battery_state = SYSTEM_BATTERY_STATE()
    power_information = SYSTEM_POWER_INFORMATION()

    if power_dll.CallNtPowerInformation(5, None, 0, ctypes.pointer(battery_state), ctypes.sizeof(battery_state)) != 0:
        return None

    if power_dll.CallNtPowerInformation(12, None, 0, ctypes.pointer(power_information), ctypes.sizeof(power_information)) != 0:
        return None

    battery_info = {
        'plugged_in': True if system_power_status.ACLineStatus == 1 else False,
        'battery_flag': system_power_status.BatteryFlag,
        'battery_life_percent': system_power_status.BatteryLifePercent,
        'battery_life_time': system_power_status.BatteryLifeTime if system_power_status.BatteryLifeTime != 4294967295 else -1,
        'battery_full_life_time': system_power_status.BatteryFullLifeTime if system_power_status.BatteryFullLifeTime != 4294967295 else -1,
        'battery_rate': battery_state.Rate,
        'max_idleness_allowed': power_information.MaxIdlenessAllowed,
        'idleness': power_information.Idleness,
        'cooling_mode': power_information.CoolingMode
    }
    return battery_info
