# Copyright (c) 2022, Bamboooz
# All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import os

GB = 1024 * 1024 * 1024


# returns the total disk space in GB for a specified drive letter
def get_total_space(drive_letter):
    # Get the total number of bytes on the drive and convert it to GB
    total_bytes = os.statvfs(drive_letter).f_frsize * os.statvfs(drive_letter).f_blocks
    return round(total_bytes / GB, 2)


# returns the used disk space in GB for a specified drive letter
def get_used_space(drive_letter):
    # Get the number of used bytes on the drive and convert it to GB
    used_bytes = os.statvfs(drive_letter).f_frsize * (
                os.statvfs(drive_letter).f_blocks - os.statvfs(drive_letter).f_bfree)
    return round(used_bytes / GB, 2)


# returns the free disk space in GB for a specified drive letter
def get_free_space(drive_letter):
    # Get the number of free bytes on the drive and convert it to GB
    free_bytes = os.statvfs(drive_letter).f_frsize * os.statvfs(drive_letter).f_bfree
    return round(free_bytes / GB, 2)


# returns the used disk space percentage for a specified drive letter
def get_used_space_percent(drive_letter):
    # Calculate the used disk space percentage
    used_percent = (1 - (os.statvfs(drive_letter).f_bfree / os.statvfs(drive_letter).f_blocks)) * 100
    return round(used_percent, 2)


# returns a list of mounted drive paths on a Linux system
def get_drive_list():
    with open("/proc/mounts", "r") as f:
        # Read the file and return a list of the second column of each line (which contains the drive path)
        return [line.split()[1] for line in f.readlines()]
